// tslint:disable:comment-format binary-expression-operand-order max-line-length

//! moment.js locale configuration
//! locale : Persian [fa-IR]
//! author : reza faghani: https://github.com/rezafaghani
//! author : mohsen mottaghizade

import { LocaleData } from '../locale/locale.class';

const symbolMap: {[key: string]: string} = {
  1: '١',
  2: '٢',
  3: '٣',
  4: '٤',
  5: '٥',
  6: '٦',
  7: '٧',
  8: '٨',
  9: '٩',
  0: '٠'
};
const numberMap: {[key: string]: string} = {
  '١': '1',
  '٢': '2',
  '٣': '3',
  '٤': '4',
  '٥': '5',
  '٦': '6',
  '٧': '7',
  '٨': '8',
  '٩': '9',
  '٠': '0'
};
const pluralForm = function (num: number): number {
  return num === 0 ? 0 : num === 1 ? 1 : num === 2 ? 2 : num % 100 >= 3 && num % 100 <= 10 ? 3 : num % 100 >= 11 ? 4 : 5;
};
const plurals: {[key: string]: [string, string, [string, string], string, string, string]} = {
  s: ['کمتراز یک ثانیه', 'یک ثانیه', ['دوثانیه', 'دوثانیه'], '%d ثانیه', '%d ثانية', '%d ثانية'],
  m: ['کمتراز یک دقیقه', 'یک دقیقه', ['دو دقیقه', 'دو دقیقه'], '%d دقیقه ها', '%d دقیقه', '%d دقیقه'],
  h: ['کمتر از یک ساعت', 'یک ساعت', ['دوساعت', 'دوساعت'], '%d ساعت ها', '%d ساعت', '%d ساعت'],
  d: ['کمتر از یک روز', 'یک روز', ['دوروز', 'دوروز'], '%d روزها', '%d روز', '%d روز'],
  M: ['کمتر از یک ماه', 'یک ماه', ['دو ماه', 'دوماه'], '%d ماه ها', '%d شماه', '%d ماه'],
  y: ['کمتر از یک سال', 'یک سال', ['دوسال', 'دوسال'], '%d سال ها', '%d سال', '%d سال']
};
const pluralize = function (u: string) {
  return function (num: number, withoutSuffix: boolean): string {
    const f = pluralForm(num);
    let str = plurals[u][pluralForm(num)];
    if (f === 2) {
      str = str[withoutSuffix ? 0 : 1];
    }

    return (str as string).replace(/%d/i, num.toString());
  };
};
const months: string[] = [
  'بهمن',
  'اسفند',
  'فروردین',
  'اردیبهشت',
  'خرداد',
  'تیر',
  'مرداد',
  'شهریور',
  'مهر',
  'آبان',
  'آذر',
  'دی'
];

export const arLocale: LocaleData = {
  abbr: 'fa-IR',
  months,
  monthsShort: months,
  weekdays: 'شنبه_یکشنبه_دوشنبه_سه شنبه_چهارشنبه_پنجشنبه_جمعه'.split('_'),
  weekdaysShort: 'شنبه_یکشنبه_دوشنبه_سه شنبه_چهارشنبه_پنجشنبه_جمعه'.split('_'),
  weekdaysMin: 'ش_ی_د_س_چ_پ_ج'.split('_'),
  weekdaysParseExact: true,
  longDateFormat: {
    LT: 'HH:mm',
    LTS: 'HH:mm:ss',
    L: 'D/\u200FM/\u200FYYYY',
    LL: 'D MMMM YYYY',
    LLL: 'D MMMM YYYY HH:mm',
    LLLL: 'dddd D MMMM YYYY HH:mm'
  },
  meridiemParse: /ص|م/,
  isPM(input) {
    return 'م' === input;
  },
  meridiem(hour, minute, isLower) {
    if (hour < 12) {
      return 'ص';
    } else {
      return 'م';
    }
  },
  calendar: {
    sameDay: '[امروز] LT',
    nextDay: '[روز بعدی] LT',
    nextWeek: 'dddd [هفته بعدی] LT',
    lastDay: '[روز قبل] LT',
    lastWeek: 'dddd [هفته قبل] LT',
    sameElse: 'L'
  },
  relativeTime: {
    future: 'بعد %s',
    past: 'پیش %s',
    s: pluralize('s'),
    ss: pluralize('s'),
    m: pluralize('m'),
    mm: pluralize('m'),
    h: pluralize('h'),
    hh: pluralize('h'),
    d: pluralize('d'),
    dd: pluralize('d'),
    M: pluralize('M'),
    MM: pluralize('M'),
    y: pluralize('y'),
    yy: pluralize('y')
  },
  preparse(str: string): string {
    return str.replace(/[١٢٣٤٥٦٧٨٩٠]/g, function (match) {
      return numberMap[match];
    }).replace(/،/g, ',');
  },
  postformat(str: string) {
    return str.replace(/\d/g, function (match) {
      return symbolMap[match];
    }).replace(/,/g, '،');
  },
  week: {
    dow: 6, // Saturday is the first day of the week.
    doy: 12  // The week that contains Jan 1st is the first week of the year.
  }
};
